{% extends "carrot/base_site.html" %}

{% block content %}
    {% el "h2" object %}
    {% ot "div" class="task-info" %}
        {% ot "form" action="" method="post" %}
            {% csrf_token %}
            {% el "p" "Are you sure you want to delete this task?" %}
            {% el "input" type="submit" class="btn_standard left" value="Confirm" %}
        {% ct "form" %}
    {% ct "div" %}
{% endblock %}