{% extends "carrot/base_site.html" %}

{% block content %}
    {% el "h2" "Queued tasks" %}
    {% ot "div" class="container" %}
        {% task_queue object_list %}
        {{ object_list|table_strapline }}
    {% ct "div" %}
    {% el "h2" "Failed tasks" %}
    {% ot "div" class="container" %}
        {% failed_task_queue failed_tasks %}
        {{ failed_tasks|table_strapline_failed }}
    {% ct "div" %}
    {% el "h2" "Completed tasks" %}
    {% ot "div" class="container" %}
        {% completed_task_queue completed_tasks %}
        {{ completed_tasks|table_strapline_completed }}
    {% ct "div" %}
    {% el "h2" "Scheduled tasks" %}
    {% ot "div" class="container" %}
        {% scheduled_task_queue scheduled_tasks %}
        {% strapline_with_url "" %}
    {% ct "div" %}
{% endblock %}